package main

import (
	"net"

	"gitlab.mashplace.com/mashplace/backend/common/api/health"
	"gitlab.mashplace.com/mashplace/backend/common/generic/addr"
	"gitlab.mashplace.com/mashplace/backend/common/generic/grpc"
	"gitlab.mashplace.com/mashplace/backend/common/generic/log"
	"gitlab.mashplace.com/mashplace/backend/common/generic/tracer"
	"gitlab.mashplace.com/mashplace/backend/gateway/impl"
	"gitlab.mashplace.com/mashplace/backend/gateway/version"
)

func main() {
	defer tracer.Close()

	listener, err := net.Listen("tcp", addr.Grpc)
	if err != nil {
		panic(err)
	}

	grpcServer, err := grpc.NewServer("gateway")
	if err != nil {
		panic(err)
	}

	healthServer := version.NewHealthServer()
	health.RegisterHealthServiceServer(grpcServer, healthServer)

	go impl.Serve(addr.HTTP)

	log.Infoln("gateway started")

	err = grpcServer.Serve(listener)
	if err != nil {
		panic(err)
	}
}
