package connections

import (
	"context"

	googleGrpc "google.golang.org/grpc"

	account "gitlab.mashplace.com/mashplace/backend/account/api"
	"gitlab.mashplace.com/mashplace/backend/common/generic/env"
	"gitlab.mashplace.com/mashplace/backend/common/generic/grpc"
)

var (
	AccountServiceURL = env.ResolveString("ACCOUNT_SERVICE_URL", "mashroom-account:10000")
	AccountService    = &accountServiceCache{
		AccountServiceClient: account.NewAccountServiceClient(grpc.Dial(AccountServiceURL, "Account")),
	}
)

type accountServiceCache struct {
	account.AccountServiceClient
}

func (cache *accountServiceCache) Get(
	ctx context.Context,
	req *account.GetRequest,
	opts ...googleGrpc.CallOption,
) (*account.GetResponse, error) {
	res, err := cacheMethod(ctx, "account", req, func() (interface{}, error) {
		return cache.AccountServiceClient.Get(ctx, req, opts...)
	})
	if err != nil {
		return nil, err
	}

	v, _ := res.(*account.GetResponse)

	return v, nil
}
