package connections

import (
	"context"
	"encoding/json"

	"gitlab.mashplace.com/mashplace/backend/gateway/impl/gwctx"
)

type cacheResult struct {
	value interface{}
	error error
}

func cacheMethod(
	ctx context.Context,
	ns string,
	req interface{},
	caller func() (interface{}, error),
) (interface{}, error) {
	opctx := gwctx.OperationContext(ctx)
	if opctx == nil {
		return caller()
	}

	bs, err := json.Marshal(req)
	if err != nil {
		return caller()
	}

	k := ns + "-" + string(bs)

	cached, ok := opctx.Value(k).(cacheResult)
	if !ok {
		resv, reserr := caller()
		cached = cacheResult{value: resv, error: reserr}

		opctx.Set(k, cached)
	}

	return cached.value, cached.error
}
