package connections

import (
	"context"

	googleGrpc "google.golang.org/grpc"

	offer "gitlab.mashplace.com/mashplace/backend/offer/api"

	"gitlab.mashplace.com/mashplace/backend/common/generic/env"
	"gitlab.mashplace.com/mashplace/backend/common/generic/grpc"
)

var (
	OfferServiceURL = env.ResolveString("OFFER_SERVICE_URL", "mashroom-offer:10000")
	OfferService    = &offerServiceCache{
		OfferServiceClient: offer.NewOfferServiceClient(grpc.Dial(OfferServiceURL, "Offer")),
	}
)

type offerServiceCache struct {
	offer.OfferServiceClient
}

func (cache *offerServiceCache) Get(
	ctx context.Context,
	req *offer.GetRequest,
	opts ...googleGrpc.CallOption,
) (*offer.GetResponse, error) {
	res, err := cacheMethod(ctx, "offer", req, func() (interface{}, error) {
		return cache.OfferServiceClient.Get(ctx, req, opts...)
	})
	if err != nil {
		return nil, err
	}

	v, _ := res.(*offer.GetResponse)

	return v, nil
}
