package connections

import (
	"context"

	googleGrpc "google.golang.org/grpc"

	"gitlab.mashplace.com/mashplace/backend/common/generic/env"
	"gitlab.mashplace.com/mashplace/backend/common/generic/grpc"
	property "gitlab.mashplace.com/mashplace/backend/property/api"
)

var (
	PropertyServiceURL = env.ResolveString("PROPERTY_SERVICE_URL", "mashroom-property:10000")
	PropertyService    = &propertyServiceCache{
		PropertyServiceClient: property.NewPropertyServiceClient(grpc.Dial(PropertyServiceURL, "Property")),
	}
)

type propertyServiceCache struct {
	property.PropertyServiceClient
}

func (cache *propertyServiceCache) Get(
	ctx context.Context,
	req *property.GetRequest,
	opts ...googleGrpc.CallOption,
) (*property.GetResponse, error) {
	res, err := cacheMethod(ctx, "property", req, func() (interface{}, error) {
		return cache.PropertyServiceClient.Get(ctx, req, opts...)
	})
	if err != nil {
		return nil, err
	}

	v, _ := res.(*property.GetResponse)

	return v, nil
}
