// Package recaptcha represents client side logic to assessments recaptcha enterprise tokens
package recaptcha

import (
	"context"
	"fmt"

	"gitlab.mashplace.com/mashplace/backend/common/api/grpcerr"
	"gitlab.mashplace.com/mashplace/backend/common/generic/env"
	"gitlab.mashplace.com/mashplace/backend/common/generic/log"
	"gitlab.mashplace.com/mashplace/backend/gateway/impl/recaptcha/credentials"
	"google.golang.org/api/option"

	recaptcha "cloud.google.com/go/recaptchaenterprise/v2/apiv1"
	recaptchapb "cloud.google.com/go/recaptchaenterprise/v2/apiv1/recaptchaenterprisepb"
)

var client *recaptcha.Client

var RecaptchaSiteKey = env.ResolveString("RECAPTCHA_SITE_KEY", "6LfnLE4dAAAAAOhHpLIsLtTewotYBPli6E7MLq8f")
var RecaptchaProjectID = env.ResolveString("RECAPTCHA_PROJECT_ID", "robotic-charmer-217508")

var ErrRecaptchaTokenIsRequired = grpcerr.New(grpcerr.InvalidData, "recaptcha token is required")
var errorRecaptchaTokenInvalid = grpcerr.New(grpcerr.InvalidData, "recaptcha token is invalid")
var errorRiskAnalysisScoreLow = grpcerr.New(grpcerr.NoAuth, "risk analysis score is low, access is denied")

/*var errorActionAttributesAreNotMatched = grpcerr.New(grpcerr.InvalidData, "action attribute in recaptcha "+
"is not matched in expecting to score")*/

func init() {
	jsonData, err := credentials.FS.ReadFile("robotic-charmer-217508-8e5a9b466805.json")
	if err != nil {
		panic(err)
	}

	ctx := context.Background()

	client, err = recaptcha.NewClient(ctx, option.WithCredentialsJSON(jsonData))
	if err != nil {
		panic(err)
	}
}

func CheckRecaptchaToken(ctx context.Context, token string) error {
	request := &recaptchapb.CreateAssessmentRequest{
		Assessment: &recaptchapb.Assessment{
			Event: &recaptchapb.Event{
				Token:   token,
				SiteKey: RecaptchaSiteKey,
			},
		},
		Parent: fmt.Sprintf("projects/%s", RecaptchaProjectID),
	}

	response, err := client.CreateAssessment(ctx, request)
	if err != nil {
		return err
	}

	if response.TokenProperties.Valid == false {
		log.Infof("The CreateAssessment() call failed because the token"+
			" was invalid for the following reasons: %v",
			response.TokenProperties.InvalidReason)
		return errorRecaptchaTokenInvalid
	}

	if response.RiskAnalysis.Score <= 0.3 {
		return errorRiskAnalysisScoreLow
	}

	return nil
}
