package resolvers

import (
	"github.com/graphql-go/graphql"
	address "gitlab.mashplace.com/mashplace/backend/address/api"
	"gitlab.mashplace.com/mashplace/backend/common/graphql/util"
	"gitlab.mashplace.com/mashplace/backend/gateway/impl/connections"
	"gitlab.mashplace.com/mashplace/backend/gateway/impl/middleware"
)

func initAddress(rootQueries, rootMutations *graphql.Fields) {
	query := graphql.NewObject(middleware.GraphqlMiddlewareObject(graphql.ObjectConfig{
		Name:        "AddressQuery",
		Description: "Address query root",
		Fields: graphql.Fields{
			"postcode": &graphql.Field{
				Description: "Get addresses by postcode",
				Args:        util.BindArguments(address.PostcodeRequest{}),
				Resolve:     AddressPostcodeResolver,
				Type:        util.BindOutput(address.AddressesResponse{}, "Addresses array", nil),
			},
			"query": &graphql.Field{
				Description: "Get addresses by query ",
				Args:        util.BindArguments(address.QueryRequest{}),
				Resolve:     AddressQueryResolver,
				Type:        util.BindOutput(address.AddressesResponse{}, "Addresses array", nil),
			},
			"autocomplete": &graphql.Field{
				Description: "Autocomplete address",
				Args:        util.BindArguments(address.QueryRequest{}),
				Resolve:     AddressAutocompleteResolver,
				Type: util.BindOutput(
					address.AutocompleteAddressResponse{},
					"Autocomplete address response",
					nil,
				),
			},
			"udprnurl": &graphql.Field{
				Description: "Get address by udprn url ",
				Args:        util.BindArguments(address.UdprnRequest{}),
				Resolve:     AddressUdprnResolver,
				Type:        util.BindOutput(address.UdprnResponse{}, "Address response by udprn url", nil),
			},
		},
	}))

	(*rootQueries)["address"] = &graphql.Field{
		Type: query,
		Resolve: func(p graphql.ResolveParams) (interface{}, error) {
			return query, nil
		},
	}
}

func AddressPostcodeResolver(params graphql.ResolveParams) (interface{}, error) {
	request := &address.PostcodeRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	return connections.AddressService.GetAddressesByPostcode(params.Context, request)
}

func AddressQueryResolver(params graphql.ResolveParams) (interface{}, error) {
	request := &address.QueryRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	return connections.AddressService.GetAddressesByQuery(params.Context, request)
}

func AddressAutocompleteResolver(params graphql.ResolveParams) (interface{}, error) {
	request := &address.QueryRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	return connections.AddressService.AutocompleteAddressByQuery(params.Context, request)
}

func AddressUdprnResolver(params graphql.ResolveParams) (interface{}, error) {
	request := &address.UdprnRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	return connections.AddressService.GetAddressByUdprn(params.Context, request)
}
