package admin

import (
	"github.com/graphql-go/graphql"
	comments "gitlab.mashplace.com/mashplace/backend/comments/api"
	q "gitlab.mashplace.com/mashplace/backend/common/api/generic/query"
	"gitlab.mashplace.com/mashplace/backend/common/graphql/util"
	"gitlab.mashplace.com/mashplace/backend/gateway/impl/connections"
	"gitlab.mashplace.com/mashplace/backend/gateway/impl/resolvers"
)

func initComments(query, mutation *graphql.Fields) {
	(*query)["allComments"] = &graphql.Field{
		Description: "List all comments",
		Args:        util.BindArguments(allParams{}),
		Resolve:     adminQueryAllCommentsResolver,
		Type:        util.BindOutput(comments.CommentFilterResponse{}, "Comments filtered list", nil),
	}
	(*mutation)["updateComment"] = &graphql.Field{
		Description: "Update comment",
		Args:        util.BindArguments(comments.ServiceCommentUpdateRequest{}),
		Resolve:     adminMutationUpdateCommentResolver,
		Type:        util.BindOutput(comments.ServiceCommentUpdateResponse{}, "Update comment response", nil),
	}
}

func adminQueryAllCommentsResolver(params graphql.ResolveParams) (interface{}, error) {
	if err := resolvers.CheckPermissions(params.Context, resolvers.AdminRole); err != nil {
		return nil, err
	}

	input := &allParams{}

	if err := util.UnmarshalArguments(params.Args, input); err != nil {
		return nil, err
	}

	req := &q.QueryParams{
		Filter: input.Filter,
		Sort:   input.Sort,
		Pagination: &q.QueryParamsPagination{
			Page:    input.Pagination.Page,
			PerPage: input.Pagination.PerPage,
		},
	}

	if len(input.Sort) == 0 {
		req.Sort = []*q.QueryParamsSort{{Field: "created", Order: -1}}
	}

	return connections.CommentsService.CommentFilter(params.Context, &comments.CommentFilterRequest{
		Params: req,
	})
}

func adminMutationUpdateCommentResolver(params graphql.ResolveParams) (interface{}, error) {
	if err := resolvers.CheckPermissions(params.Context, resolvers.AdminRole); err != nil {
		return nil, err
	}

	req := &comments.ServiceCommentUpdateRequest{}
	if err := util.UnmarshalArguments(params.Args, req); err != nil {
		return nil, err
	}

	return connections.CommentsService.ServiceCommentUpdate(params.Context, req)
}
