package admin

import (
	"os"

	"github.com/graphql-go/graphql"
	"gitlab.mashplace.com/mashplace/backend/common/graphql/util"
	"gitlab.mashplace.com/mashplace/backend/gateway/impl/connections"
	"gitlab.mashplace.com/mashplace/backend/gateway/impl/resolvers"
	geo "gitlab.mashplace.com/mashplace/backend/geo/api"
)

func initGeo(query, mutation *graphql.Fields) {
	(*mutation)["generateSitemaps"] = &graphql.Field{
		Description: "Generate sitemaps",
		Args:        util.BindArguments(geo.GenerateSitemapsRequest{}),
		Resolve:     adminMutationGenerateSitemapsResolver,
		Type:        util.BindOutput(geo.GenerateSitemapsResponse{}, "Generate sitemaps response", nil),
	}

	(*mutation)["clearSitemapsCache"] = &graphql.Field{
		Description: "Clear sitemaps cache",
		Args:        util.BindArguments(geo.ClearSitemapsCacheRequest{}),
		Resolve:     adminMutationClearSitemapsCacheResolver,
		Type:        util.BindOutput(geo.ClearSitemapsCacheResponse{}, "Clear sitemaps cache response", nil),
	}
}

func adminMutationGenerateSitemapsResolver(params graphql.ResolveParams) (interface{}, error) {
	if err := resolvers.CheckPermissions(params.Context, resolvers.AdminRole); err != nil {
		return nil, err
	}
	request := &geo.GenerateSitemapsRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	return connections.GeoService.GenerateSitemaps(params.Context, request)
}

func adminMutationClearSitemapsCacheResolver(params graphql.ResolveParams) (interface{}, error) {
	if err := resolvers.CheckPermissions(params.Context, resolvers.AdminRole); err != nil {
		return nil, err
	}
	request := &geo.ClearSitemapsCacheRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	err := os.RemoveAll(SitemapCacheDir)
	if err != nil {
		return nil, err
	}
	return connections.GeoService.ClearSitemapsCache(params.Context, request)
}
