package admin

import (
	"gitlab.mashplace.com/mashplace/backend/common/graphql/util"
	"gitlab.mashplace.com/mashplace/backend/gateway/impl/connections"
	"gitlab.mashplace.com/mashplace/backend/gateway/impl/resolvers"
	messaging "gitlab.mashplace.com/mashplace/backend/messaging/api"

	"github.com/graphql-go/graphql"
)

func initMessaging(_, mutation *graphql.Fields) {
	(*mutation)["deleteMessagesByUserID"] = &graphql.Field{
		Description: "Delete messages by user id",
		Args:        util.BindArguments(messaging.DeleteMessagesByUserIDRequest{}),
		Resolve:     adminMutationDeleteMessagesByUserIDResolver,
		Type: util.BindOutput(messaging.DeleteMessagesByUserIDResponse{},
			"Delete messages by user id response", nil),
	}
}

func adminMutationDeleteMessagesByUserIDResolver(params graphql.ResolveParams) (interface{}, error) {
	if err := resolvers.CheckPermissions(params.Context, resolvers.AdminRole); err != nil {
		return nil, err
	}
	request := &messaging.DeleteMessagesByUserIDRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	resp, err := connections.MessagingService.DeleteMessagesByUserID(params.Context, request)
	if err != nil {
		return nil, err
	}

	return resp, nil
}
