package admin

import (
	"github.com/graphql-go/graphql"
	"gitlab.mashplace.com/mashplace/backend/common/graphql/util"
	"gitlab.mashplace.com/mashplace/backend/gateway/impl/connections"
	"gitlab.mashplace.com/mashplace/backend/gateway/impl/resolvers"
	invoice "gitlab.mashplace.com/mashplace/backend/invoice/api"
)

func initOrder(query, mutation *graphql.Fields) {
	(*query)["products"] = &graphql.Field{
		Description: "List all property products",
		Args:        util.BindArguments(invoice.ProductListRequest{}),
		Resolve:     adminQueryProductsResolver,
		Type:        util.BindOutput(invoice.ProductListResponse{}, "Product list", nil),
	}
	(*query)["allOrders"] = &graphql.Field{
		Description: "List orders",
		Args:        util.BindArguments(invoice.OrderListRequest{}),
		Resolve:     adminQueryOrdersResolver,
		Type:        util.BindOutput(invoice.OrderListResponse{}, "Orders list", nil),
	}
	(*mutation)["saveOrder"] = &graphql.Field{
		Description: "Save order",
		Args:        util.BindArguments(invoice.OrderSaveRequest{}),
		Resolve:     adminMutationSaveOrderResolver,
		Type:        util.BindOutput(invoice.OrderSaveResponse{}, "Save status", nil),
	}
	(*mutation)["commitOrder"] = &graphql.Field{
		Description: "Save order",
		Args:        util.BindArguments(invoice.OrderCommitRequest{}),
		Resolve:     adminMutationCommitOrderResolver,
		Type:        util.BindOutput(invoice.OrderCommitResponse{}, "Commit status", nil),
	}
	(*mutation)["createRemoteSalePayment"] = &graphql.Field{
		Description: "Create order/invoice/payment for remote sale",
		Args:        util.BindArguments(invoice.CreateRemoteSalePaymentRequest{}),
		Resolve:     adminMutationCreateRemoteSalePaymentResolver,
		Type:        util.BindOutput(invoice.CreateRemoteSalePaymentResponse{}, "Commit status", nil),
	}
}

func adminQueryProductsResolver(params graphql.ResolveParams) (interface{}, error) {
	if err := resolvers.CheckPermissions(params.Context, resolvers.AdminRole); err != nil {
		return nil, err
	}

	request := &invoice.ProductListRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}

	return connections.InvoiceService.ProductList(params.Context, request)
}

func adminQueryOrdersResolver(params graphql.ResolveParams) (interface{}, error) {
	if err := resolvers.CheckPermissions(params.Context, resolvers.AdminRole); err != nil {
		return nil, err
	}

	request := &invoice.OrderListRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}

	return connections.InvoiceService.OrderList(params.Context, request)
}

func adminMutationCreateRemoteSalePaymentResolver(params graphql.ResolveParams) (interface{}, error) {
	if err := resolvers.CheckPermissions(params.Context, resolvers.AdminRole); err != nil {
		return nil, err
	}

	request := &invoice.CreateRemoteSalePaymentRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}

	return connections.InvoiceService.CreateRemoteSalePayment(params.Context, request)
}

func adminMutationSaveOrderResolver(params graphql.ResolveParams) (interface{}, error) {
	if err := resolvers.CheckPermissions(params.Context, resolvers.AdminRole); err != nil {
		return nil, err
	}

	request := &invoice.OrderSaveRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}

	return connections.InvoiceService.OrderSave(params.Context, request)
}

func adminMutationCommitOrderResolver(params graphql.ResolveParams) (interface{}, error) {
	if err := resolvers.CheckPermissions(params.Context, resolvers.AdminRole); err != nil {
		return nil, err
	}

	request := &invoice.OrderCommitRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}

	return connections.InvoiceService.OrderCommit(params.Context, request)
}
