package admin

import (
	"github.com/graphql-go/graphql"
	account "gitlab.mashplace.com/mashplace/backend/account/api"
	auth "gitlab.mashplace.com/mashplace/backend/auth/api"
	"gitlab.mashplace.com/mashplace/backend/common/graphql/util"
	ws "gitlab.mashplace.com/mashplace/backend/common/graphql/websocket"
	"gitlab.mashplace.com/mashplace/backend/gateway/impl/connections"
	"gitlab.mashplace.com/mashplace/backend/gateway/impl/resolvers"
)

func initOwnership(query, mutation *graphql.Fields) {
	(*mutation)["updateOwnershipVerification"] = &graphql.Field{
		Description: "Update ownership verification",
		Args:        util.BindArguments(account.UpdateOwnershipVerificationRequest{}),
		Resolve:     updateOwnershipVerificationResolver,
		Type:        util.BindOutput(account.UpdateOwnershipVerificationResponse{}, "Update ownership", nil),
	}

	(*query)["getOwnerships"] = &graphql.Field{
		Description: "Returns ownership list",
		Args:        util.BindArguments(account.GetOwnershipVerificationRequest{}),
		Resolve:     getOwnershipVerificationsResolver,
		Type:        util.BindOutput(account.GetOwnershipVerificationResponse{}, "Ownership list", nil),
	}
}

func updateOwnershipVerificationResolver(params graphql.ResolveParams) (interface{}, error) {
	if err := resolvers.CheckPermissions(params.Context, resolvers.AdminRole); err != nil {
		return nil, err
	}

	request := &account.UpdateOwnershipVerificationRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}

	user := params.Context.Value(ws.CtxUser).(*auth.AuthResponse)
	request.VerifiedById = user.GetAccount().GetIdValue()

	return connections.AccountService.UpdateOwnershipVerification(params.Context, request)
}

func getOwnershipVerificationsResolver(params graphql.ResolveParams) (interface{}, error) {
	if err := resolvers.CheckPermissions(params.Context, resolvers.AdminRole); err != nil {
		return nil, err
	}

	req := &account.GetOwnershipVerificationRequest{}

	if err := util.UnmarshalArguments(params.Args, req); err != nil {
		return nil, err
	}

	return connections.AccountService.GetOwnershipVerifications(params.Context, req)
}
