package admin

import (
	"github.com/graphql-go/graphql"
	"gitlab.mashplace.com/mashplace/backend/common/graphql/util"
	"gitlab.mashplace.com/mashplace/backend/gateway/impl/connections"
	"gitlab.mashplace.com/mashplace/backend/gateway/impl/resolvers"
	signature "gitlab.mashplace.com/mashplace/backend/signature/api"
)

func initSignature(query, mutation *graphql.Fields) {
	(*mutation)["updateSignature"] = &graphql.Field{
		Description: "Update signature state",
		Args:        util.BindArguments(signature.AdminRequest{}),
		Resolve:     SignatureAdminResolver,
		Type:        util.BindOutput(signature.AdminResponse{}, "Signature update result", nil),
	}
}

func SignatureAdminResolver(params graphql.ResolveParams) (interface{}, error) {
	if err := resolvers.CheckPermissions(params.Context, resolvers.AdminRole); err != nil {
		return nil, err
	}

	request := &signature.AdminRequest{}

	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}

	var status signature.SigningStatus

	switch {
	case request.GetIsDeclined():
		status = signature.SigningStatus_SIGNING_STATUS_DECLINED
	case request.GetIsComplete():
		status = signature.SigningStatus_SIGNING_STATUS_COMPLETE
	case request.GetHasError():
		status = signature.SigningStatus_SIGNING_STATUS_ERROR
	default:
		status = signature.SigningStatus_SIGNING_STATUS_PENDING
	}

	_, err := connections.SignatureService.ServiceSigningStatus(
		params.Context,
		&signature.ServiceSigningStatusRequest{
			SignatureRequestId: request.GetSignatureRequestId(),
			EventType:          request.GetEventType(),
			Status:             status,
			DocumentFileId:     request.GetMetadata().GetContractId(),
			Metadata: &signature.SigningMetadata{
				OfferId:    request.GetMetadata().GetOfferId(),
				PropertyId: request.GetMetadata().GetPropertyId(),
				ViewingId:  request.GetMetadata().GetViewingId(),
			},
		},
	)
	if err != nil {
		return nil, err
	}

	return &signature.AdminResponse{
		Success: true,
	}, nil
}
