package admin

import (
	"github.com/graphql-go/graphql"
	"gitlab.mashplace.com/mashplace/backend/common/graphql/util"
	"gitlab.mashplace.com/mashplace/backend/gateway/impl/connections"
	"gitlab.mashplace.com/mashplace/backend/gateway/impl/resolvers"
	stats "gitlab.mashplace.com/mashplace/backend/stats/api"
)

func initStats(query, mutation *graphql.Fields) {
	(*mutation)["statsUpload"] = &graphql.Field{
		Description: "Run upload",
		Args:        util.BindArguments(stats.UploadRequest{}),
		Resolve:     adminMutationUploadResolver,
		Type:        util.BindOutput(stats.UploadResponse{}, "Upload status", nil),
	}
	(*mutation)["accountUpload"] = &graphql.Field{
		DeprecationReason: "use statsUpload",
		Description:       "Run upload accounts",
		Args:              util.BindArguments(stats.UploadRequest{}),
		Resolve:           adminMutationAccountUploadResolver,
		Type:              util.BindOutput(stats.UploadResponse{}, "Upload status", nil),
	}
	(*mutation)["offerUpload"] = &graphql.Field{
		DeprecationReason: "use statsUpload",
		Description:       "Run upload offers",
		Args:              util.BindArguments(stats.UploadRequest{}),
		Resolve:           adminMutationOfferUploadResolver,
		Type:              util.BindOutput(stats.UploadResponse{}, "Upload status", nil),
	}
	(*mutation)["propertyUpload"] = &graphql.Field{
		DeprecationReason: "use statsUpload",
		Description:       "Run upload properties",
		Args:              util.BindArguments(stats.UploadRequest{}),
		Resolve:           adminMutationPropertyUploadResolver,
		Type:              util.BindOutput(stats.UploadResponse{}, "Upload status", nil),
	}
}

func adminMutationUploadResolver(params graphql.ResolveParams) (interface{}, error) {
	if err := resolvers.CheckPermissions(params.Context, resolvers.AdminRole); err != nil {
		return nil, err
	}

	request := &stats.UploadRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}

	resp, err := connections.StatsService.Upload(params.Context, request)
	if err != nil {
		return nil, err
	}

	return resp, nil
}

func adminMutationAccountUploadResolver(params graphql.ResolveParams) (interface{}, error) {
	if err := resolvers.CheckPermissions(params.Context, resolvers.AdminRole); err != nil {
		return nil, err
	}

	request := &stats.UploadRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}

	request.StatsType = stats.StatsType_ACCOUNT

	resp, err := connections.StatsService.Upload(params.Context, request)
	if err != nil {
		return nil, err
	}

	return resp, nil
}

func adminMutationOfferUploadResolver(params graphql.ResolveParams) (interface{}, error) {
	if err := resolvers.CheckPermissions(params.Context, resolvers.AdminRole); err != nil {
		return nil, err
	}

	request := &stats.UploadRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}

	request.StatsType = stats.StatsType_OFFER

	resp, err := connections.StatsService.Upload(params.Context, request)
	if err != nil {
		return nil, err
	}

	return resp, nil
}

func adminMutationPropertyUploadResolver(params graphql.ResolveParams) (interface{}, error) {
	if err := resolvers.CheckPermissions(params.Context, resolvers.AdminRole); err != nil {
		return nil, err
	}

	request := &stats.UploadRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}

	request.StatsType = stats.StatsType_PROPERTY

	resp, err := connections.StatsService.Upload(params.Context, request)
	if err != nil {
		return nil, err
	}

	return resp, nil
}
