package admin

import (
	"github.com/graphql-go/graphql"
	account "gitlab.mashplace.com/mashplace/backend/account/api"
	auth "gitlab.mashplace.com/mashplace/backend/auth/api"
	"gitlab.mashplace.com/mashplace/backend/common/graphql/util"
	ws "gitlab.mashplace.com/mashplace/backend/common/graphql/websocket"
	"gitlab.mashplace.com/mashplace/backend/gateway/impl/connections"
	"gitlab.mashplace.com/mashplace/backend/gateway/impl/resolvers"
)

func initVerification(query, mutation *graphql.Fields) {
	(*mutation)["createAdminVerificationSession"] = &graphql.Field{
		Description: "Create admin verification session",
		Args:        util.BindArguments(account.CreateAdminVerificationSessionRequest{}),
		Resolve:     createAdminVerificationSessionResolver,
		Type:        util.BindOutput(account.CreateAdminVerificationSessionResponse{}, "Create admin session", nil),
	}

	(*mutation)["updateAdminVerificationSession"] = &graphql.Field{
		Description: "Update admin verification session",
		Args:        util.BindArguments(account.UpdateAdminVerificationSessionRequest{}),
		Resolve:     updateAdminVerificationSessionResolver,
		Type:        util.BindOutput(account.UpdateAdminVerificationSessionResponse{}, "Update admin session", nil),
	}
}

func createAdminVerificationSessionResolver(params graphql.ResolveParams) (interface{}, error) {
	if err := resolvers.CheckPermissions(params.Context, resolvers.AdminRole); err != nil {
		return nil, err
	}

	request := &account.CreateAdminVerificationSessionRequest{}

	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}

	user := params.Context.Value(ws.CtxUser).(*auth.AuthResponse)
	request.AdminId = user.GetAccount().GetIdValue()

	return connections.AccountService.CreateAdminVerificationSession(params.Context, request)
}

func updateAdminVerificationSessionResolver(params graphql.ResolveParams) (interface{}, error) {
	if err := resolvers.CheckPermissions(params.Context, resolvers.AdminRole); err != nil {
		return nil, err
	}

	req := &account.UpdateAdminVerificationSessionRequest{}

	if err := util.UnmarshalArguments(params.Args, req); err != nil {
		return nil, err
	}

	user := params.Context.Value(ws.CtxUser).(*auth.AuthResponse)
	req.AdminId = user.GetAccount().GetIdValue()

	return connections.AccountService.UpdateAdminVerificationSession(params.Context, req)
}
