package resolvers

import (
	"github.com/graphql-go/graphql"
	closecom "gitlab.mashplace.com/mashplace/backend/closecom/api"
	"gitlab.mashplace.com/mashplace/backend/common/graphql/util"
	document "gitlab.mashplace.com/mashplace/backend/document/api"
	"gitlab.mashplace.com/mashplace/backend/gateway/impl/connections"
	"gitlab.mashplace.com/mashplace/backend/gateway/impl/middleware"
)

func initCloseCom(rootQueries, rootMutations *graphql.Fields) {
	leadQuery := graphql.NewObject(middleware.GraphqlMiddlewareObject(graphql.ObjectConfig{
		Name:        "CloseComLeadQuery",
		Description: "CloseCom lead query root",
		Fields: graphql.Fields{
			"get": &graphql.Field{
				Description: "Get lead by id",
				Args:        util.BindArguments(document.GetRequest{}),
				Resolve:     LeadQueryGetResolver,
				Type:        util.BindOutput(closecom.LeadGetResponse{}, "Lead with given id", nil),
			},
		},
	}))

	closecomQuery := graphql.NewObject(middleware.GraphqlMiddlewareObject(graphql.ObjectConfig{
		Name:        "CloseComQuery",
		Description: "CloseCom query root",
		Fields: graphql.Fields{
			"lead": &graphql.Field{
				Type: leadQuery,
				Resolve: func(p graphql.ResolveParams) (interface{}, error) {
					return leadQuery, nil
				},
			},
		},
	}))

	(*rootQueries)["closecom"] = &graphql.Field{
		Type: closecomQuery,
		Resolve: func(p graphql.ResolveParams) (interface{}, error) {
			return closecomQuery, nil
		},
	}
}

func LeadQueryGetResolver(params graphql.ResolveParams) (interface{}, error) {
	request := &closecom.LeadGetRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}

	return connections.ClosecomService.LeadGet(params.Context, request)
}
