package resolvers

import (
	"encoding/json"

	messaging "gitlab.mashplace.com/mashplace/backend/messaging/api"

	"github.com/graphql-go/graphql"
	"gitlab.mashplace.com/mashplace/backend/common/graphql/util"
	"gitlab.mashplace.com/mashplace/backend/gateway/impl/connections"
	"gitlab.mashplace.com/mashplace/backend/gateway/impl/middleware"
	notification "gitlab.mashplace.com/mashplace/backend/notification/api"
)

type QuestionRequest struct {
	Email   string `json:"email"`
	Name    string `json:"name"`
	Phone   string `json:"phone"`
	Message string `json:"message"`
}

type QuestionResponse struct {
	Success bool `json:"success"`
}

func initContact(rootQueries, rootMutations *graphql.Fields) {
	mutation := graphql.NewObject(middleware.GraphqlMiddlewareObject(graphql.ObjectConfig{
		Name:        "ContactMutation",
		Description: "Contact mutation root",
		Fields: graphql.Fields{
			"question": &graphql.Field{
				Description: "Ask a question to the support",
				Args:        util.BindArguments(QuestionRequest{}),
				Resolve:     ContactMutationQuestionResolver,
				Type:        util.BindOutput(QuestionResponse{}, "Success of sending a question", nil),
			},
		},
	}))

	(*rootMutations)["contact"] = &graphql.Field{
		Type: mutation,
		Resolve: func(p graphql.ResolveParams) (interface{}, error) {
			return mutation, nil
		},
	}
}

func ContactMutationQuestionResolver(params graphql.ResolveParams) (interface{}, error) {
	request := &QuestionRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}

	bs, _ := json.Marshal(map[string]interface{}{
		"request": request,
	})

	_, _ = connections.NotificationService.SendNotification(params.Context, &notification.SendNotificationRequest{
		Entity: notification.Entity_ENTITY_PROPERTY_QUESTION,
		Action: notification.Action_ACTION_CREATE,
		Targets: []*notification.Target{
			{
				Role: notification.Role_ROLE_ADMIN,
			},
			{
				Role: notification.Role_ROLE_AUTHOR,
				Contacts: &notification.Contacts{
					Email:    request.Email,
					Fullname: request.Name,
				},
			},
		},
		Embeds: []*notification.EventEmbed{
			{
				EmbedType:  messaging.EmbedType_EMBED_TYPE_JSON,
				EmbedValue: string(bs),
			},
		},
	})

	return &QuestionResponse{Success: true}, nil
}
