package resolvers

import (
	"github.com/graphql-go/graphql"
	"gitlab.mashplace.com/mashplace/backend/common/graphql/util"
	document "gitlab.mashplace.com/mashplace/backend/document/api"
	"gitlab.mashplace.com/mashplace/backend/gateway/impl/connections"
	"gitlab.mashplace.com/mashplace/backend/gateway/impl/middleware"
)

func initDocumentType(rootQueries *graphql.Fields) {
	documentTypeQuery := graphql.NewObject(middleware.GraphqlMiddlewareObject(graphql.ObjectConfig{
		Name:        "DocumentTypeQuery",
		Description: "DocumentType query root",
		Fields: graphql.Fields{
			"get": &graphql.Field{
				Description: "Get document type",
				Args:        util.BindArguments(document.DocumentTypeGetRequest{}),
				Resolve:     DocumentTypeQueryGetResolver,
				Type:        util.BindOutput(document.DocumentTypeGetResponse{}, "Get document type", nil),
			},
			"list": &graphql.Field{
				Description: "List document types",
				Args:        util.BindArguments(document.DocumentTypeFilterRequest{}),
				Resolve:     DocumentTypeQueryListResolver,
				Type: util.BindOutput(
					document.DocumentTypeFilterResponse{},
					"List document types",
					nil,
				),
			},
		},
	}))

	(*rootQueries)["document_type"] = &graphql.Field{
		Type: documentTypeQuery,
		Resolve: func(p graphql.ResolveParams) (interface{}, error) {
			return documentTypeQuery, nil
		},
	}
}

func DocumentTypeQueryGetResolver(params graphql.ResolveParams) (interface{}, error) {
	request := &document.DocumentTypeGetRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}

	return connections.DocumentService.DocumentTypeGet(params.Context, request)
}

func DocumentTypeQueryListResolver(params graphql.ResolveParams) (interface{}, error) {
	request := &document.DocumentTypeFilterRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}

	return connections.DocumentService.DocumentTypeFilter(params.Context, request)
}
