package enrichment

import (
	"context"
	"sync"

	"gitlab.mashplace.com/mashplace/backend/common/api/generic"

	account "gitlab.mashplace.com/mashplace/backend/account/api"
	"gitlab.mashplace.com/mashplace/backend/gateway/impl/connections"
	listing "gitlab.mashplace.com/mashplace/backend/listing/api"
	viewing "gitlab.mashplace.com/mashplace/backend/viewing/api"
)

// ResolveAccountResources resolves account owned resources once per request
func ResolveAccountResources(ctx context.Context, accounts ...*account.Account) {
	var ids []string

	accs := make(map[string]*account.AccountResources)

	for _, a := range accounts {
		if a.Resources != nil {
			continue
		}

		res := &account.AccountResources{}

		ids = append(ids, a.GetIdValue())
		accs[a.GetIdValue()] = res

		a.Resources = &account.Account_ResourcesValue{
			ResourcesValue: res,
		}
	}

	wg := &sync.WaitGroup{}

	wg.Add(2)

	go func() {
		props, err := connections.ListingService.ServicePropertyCount(ctx, &listing.ServicePropertyCountRequest{
			Selector: &listing.ServicePropertyCountRequest_Landlords{
				Landlords: &generic.StringArrayValue{Value: ids},
			},
		})
		if err != nil {
			return
		}

		for u, c := range props.GetCount() {
			accs[u].OwnedProperties = int32(c)
		}

		wg.Done()
	}()

	go func() {
		props, err := connections.ViewingService.ServiceViewingCount(ctx, &viewing.ServiceViewingCountRequest{
			Selector: &viewing.ServiceViewingCountRequest_Senders{
				Senders: &generic.StringArrayValue{Value: ids},
			},
		})
		if err != nil {
			return
		}

		for u, c := range props.GetCounts() {
			accs[u].ViewingRequests = int32(c)
		}

		wg.Done()
	}()

	wg.Wait()
}
