package legacy

import (
	property "gitlab.mashplace.com/mashplace/backend/property/api"
	tenancy "gitlab.mashplace.com/mashplace/backend/tenancy/api"
	"google.golang.org/protobuf/types/known/timestamppb"
	"google.golang.org/protobuf/types/known/wrapperspb"
)

func ConvertTenanciesLegacy(ms []*tenancy.Tenancy) (ts []*property.Tenancy) {
	for _, m := range ms {
		ts = append(ts, ConvertTenancyLegacy(m))
	}

	return
}

func ConvertTenancyLegacy(t *tenancy.Tenancy) *property.Tenancy {
	var offerid *wrapperspb.StringValue

	if t.GetLandlordIntention().GetNewOffer() != "" {
		offerid = wrapperspb.String(t.GetLandlordIntention().GetNewOffer())
	}

	var termination *timestamppb.Timestamp

	if t.GetLandlordIntention().GetChoice() == tenancy.TenancyIntentionChoice_INTENTION_TERMINATE ||
		t.GetTenantsIntention().GetChoice() == tenancy.TenancyIntentionChoice_INTENTION_TERMINATE {
		termination = t.GetEndDate()
	}

	return &property.Tenancy{
		Id:                    t.GetId(),
		Property:              wrapperspb.String(t.GetProperty()),
		Offer:                 wrapperspb.String(t.GetOffer()),
		Tenants:               convertTenancyTenantsLegacy(t.GetTenants()),
		Guarantors:            convertTenancyGuarantorsLegacy(t.GetGuarantors()),
		TenancyState:          convertTenancyStateLegacy(t.GetTenancyState()),
		LandlordIntention:     property.TenancyIntention(t.GetLandlordIntention().GetChoice()),
		TenantsIntention:      property.TenancyIntention(t.GetTenantsIntention().GetChoice()),
		IntentionNotified:     false,
		Movedin:               t.GetStartDate(),
		Movedout:              t.GetEndDate(),
		LandlordIntentionDate: t.GetLandlordIntention().GetProvided(),
		TenantsIntentionDate:  t.GetTenantsIntention().GetProvided(),
		ExpirationDate:        nil,
		TerminationDate:       termination,
		EndDate:               t.GetEndDate(),
		PenaltyConsentDate:    t.GetTenantsIntention().GetPenaltyConsented(),
		NewOffer:              offerid,
		UpdatedAt:             t.GetUpdatedAt(),
	}
}

func convertTenancyGuarantorsLegacy(ms []*tenancy.Guarantor) (gs *property.GuarantorArrayValue) {
	gs = &property.GuarantorArrayValue{}

	for _, m := range ms {
		gs.Value = append(gs.Value, &property.Guarantor{
			FirstName:    wrapperspb.String(m.GetFirstName()),
			LastName:     wrapperspb.String(m.GetLastName()),
			EmailPrimary: wrapperspb.String(m.GetEmailPrimary()),
			PhonePrimary: wrapperspb.String(m.GetPhonePrimary()),
		})
	}

	return
}

func convertTenancyTenantsLegacy(ms []*tenancy.Tenant) (ts *property.TenantArrayValue) {
	ts = &property.TenantArrayValue{}

	for _, m := range ms {
		ts.Value = append(ts.Value, &property.Tenant{
			User:     wrapperspb.String(m.GetUser()),
			Contract: wrapperspb.String(m.GetContract()),
			Active:   wrapperspb.Bool(m.GetActive()),
			Movedin:  nil,
			Movedout: nil,
			Risk:     property.TenantRisk(m.GetRisk()),
			TenantContact: &property.TenantContact{
				FirstName:    wrapperspb.String(m.GetTenantContact().GetFirstName()),
				LastName:     wrapperspb.String(m.GetTenantContact().GetLastName()),
				EmailPrimary: wrapperspb.String(m.GetTenantContact().GetEmailPrimary()),
				PhonePrimary: wrapperspb.String(m.GetTenantContact().GetPhonePrimary()),
			},
		})
	}

	return
}

func convertTenancyStateLegacy(state tenancy.TenancyState) property.TenancyState {
	if state == 0 {
		return 0
	}

	return property.TenancyState(state - 1)
}
