package legacy

import (
	"gitlab.mashplace.com/mashplace/backend/common/api/generic"
	property "gitlab.mashplace.com/mashplace/backend/property/api"
	viewing "gitlab.mashplace.com/mashplace/backend/viewing/api"
	"google.golang.org/protobuf/types/known/wrapperspb"
)

func ConvertLegacyViewingsLegacy(vs []*viewing.Viewing) (views []*property.Viewing) {
	for _, v := range vs {
		views = append(views, ConvertLegacyViewingLegacy(v))
	}

	return
}

func ConvertLegacyViewingLegacy(v *viewing.Viewing) *property.Viewing {
	var feedback []*property.ViewingFeedbackEntry

	for _, f := range v.GetFeedback().GetEntries() {
		feedback = append(feedback, &property.ViewingFeedbackEntry{
			Transition: property.ViewRequestStatus(f.GetTransition()),
			Initiator:  f.GetInitiator(),
			Comment:    wrapperspb.String(f.GetComment()),
			Rating:     wrapperspb.Int32(f.GetRating()),
		})
	}

	return &property.Viewing{
		Id:       v.GetId(),
		Property: v.GetProperty(),
		Sender: &property.Sender{
			User:           v.GetSender(),
			FirstName:      nil,
			LastName:       nil,
			EmailPrimary:   nil,
			PhonePrimary:   nil,
			BillingAddress: nil,
			Avatar:         nil,
		},
		CreatedAt: v.GetCreatedAt(),
		Status:    property.ViewRequestStatus(v.GetStatus()),
		Time: &property.ViewRequestTime{
			TimeStartsec: wrapperspb.Int32(v.GetTime().GetTimeStartsec()),
			TimeEndsec:   wrapperspb.Int32(v.GetTime().GetTimeEndsec()),
			Day:          wrapperspb.Int32(v.GetTime().GetDay()),
			Month:        wrapperspb.Int32(v.GetTime().GetMonth()),
			Year:         wrapperspb.Int32(v.GetTime().GetYear()),
		},
		Details: &property.ViewRequestDetails{
			Flatmates: &generic.StringArrayValue{
				Value: v.GetDetails().GetFlatmates(),
			},
			MoveinDate:             property.MoveinDate(v.GetDetails().GetMoveinDate()),
			HavePets:               wrapperspb.Bool(v.GetDetails().GetHavePets()),
			PetsDescription:        wrapperspb.String(v.GetDetails().GetPetsDescription()),
			HaveHouseBenefits:      wrapperspb.Bool(v.GetDetails().GetHaveHouseBenefits()),
			BenefitDetails:         wrapperspb.String(v.GetDetails().GetBenefitDetails()),
			ViewingDescription:     wrapperspb.String(v.GetDetails().GetViewingDescription()),
			CombinedHousehold:      wrapperspb.Bool(v.GetDetails().GetCombinedHousehold()),
			CountryCourtJudgements: wrapperspb.Bool(v.GetDetails().GetCountryCourtJudgements()),
			Video:                  wrapperspb.Bool(v.GetDetails().GetVideo()),
		},
		Feedback: &property.ViewingFeedback{
			Entries: feedback,
		},
		Offer:        v.GetOffer(),
		Manager:      v.GetManager(),
		RewardStatus: property.ManagerRewardStatus(v.GetRewardStatus()),
		AcceptedAt:   v.GetAcceptedAt(),
		Remotely:     v.GetRemotely(),
		UpdatedAt:    v.GetUpdatedAt(),
		Listing:      v.GetListing(),
	}
}

func ConvertViewingsLegacy(vs []*viewing.Viewing) (views []*property.ViewRequest) {
	for _, v := range vs {
		views = append(views, ConvertViewingLegacy(v))
	}

	return
}

func ConvertViewingLegacy(v *viewing.Viewing) *property.ViewRequest {
	var feedback []*property.ViewRequestFeedbackEntry

	for _, f := range v.GetFeedback().GetEntries() {
		feedback = append(feedback, &property.ViewRequestFeedbackEntry{
			Transition: property.ViewRequestStatus(f.GetTransition()),
			Initiator:  f.GetInitiator(),
			Comment:    wrapperspb.String(f.GetComment()),
			Rating:     wrapperspb.Int32(f.GetRating()),
		})
	}

	return &property.ViewRequest{
		Id:        v.GetId(),
		Property:  v.GetProperty(),
		Sender:    v.GetSender(),
		CreatedAt: v.GetCreatedAt(),
		Status:    property.ViewRequestStatus(v.GetStatus()),
		Time: &property.ViewRequestTime{
			TimeStartsec: wrapperspb.Int32(v.GetTime().GetTimeStartsec()),
			TimeEndsec:   wrapperspb.Int32(v.GetTime().GetTimeEndsec()),
			Day:          wrapperspb.Int32(v.GetTime().GetDay()),
			Month:        wrapperspb.Int32(v.GetTime().GetMonth()),
			Year:         wrapperspb.Int32(v.GetTime().GetYear()),
		},
		Details: &property.ViewRequestDetails{
			Flatmates: &generic.StringArrayValue{
				Value: v.GetDetails().GetFlatmates(),
			},
			MoveinDate:             property.MoveinDate(v.GetDetails().GetMoveinDate()),
			HavePets:               wrapperspb.Bool(v.GetDetails().GetHavePets()),
			PetsDescription:        wrapperspb.String(v.GetDetails().GetPetsDescription()),
			HaveHouseBenefits:      wrapperspb.Bool(v.GetDetails().GetHaveHouseBenefits()),
			BenefitDetails:         wrapperspb.String(v.GetDetails().GetBenefitDetails()),
			ViewingDescription:     wrapperspb.String(v.GetDetails().GetViewingDescription()),
			CombinedHousehold:      wrapperspb.Bool(v.GetDetails().GetCombinedHousehold()),
			CountryCourtJudgements: wrapperspb.Bool(v.GetDetails().GetCountryCourtJudgements()),
			Video:                  wrapperspb.Bool(v.GetDetails().GetVideo()),
		},
		Feedback: &property.ViewRequestFeedback{
			Entries: feedback,
		},
		Offer:        v.GetOffer(),
		Manager:      v.GetManager(),
		RewardStatus: property.ManagerRewardStatus(v.GetRewardStatus()),
		AcceptedAt:   v.GetAcceptedAt(),
		Remotely:     wrapperspb.Bool(v.GetRemotely()),
		UpdatedAt:    v.GetUpdatedAt(),
	}
}
