package resolvers

import (
	"github.com/graphql-go/graphql"
	account "gitlab.mashplace.com/mashplace/backend/account/api"
	auth "gitlab.mashplace.com/mashplace/backend/auth/api"
	"gitlab.mashplace.com/mashplace/backend/common/graphql/util"
	ws "gitlab.mashplace.com/mashplace/backend/common/graphql/websocket"
	"gitlab.mashplace.com/mashplace/backend/gateway/impl/connections"
	"gitlab.mashplace.com/mashplace/backend/gateway/impl/middleware"
	"gitlab.mashplace.com/mashplace/backend/gateway/impl/resolvers/errs"
)

func initOwnership(rootQueries, rootMutations *graphql.Fields) {
	query := graphql.NewObject(middleware.GraphqlMiddlewareObject(graphql.ObjectConfig{
		Name:        "OwnershipQuery",
		Description: "Ownership query root",
		Fields: graphql.Fields{
			"checkIsVerified": &graphql.Field{
				Description: "Check ownership is verified by id",
				Args:        util.BindArguments(account.OwnershipIsVerifiedRequest{}),
				Resolve:     OwnershipIsVerifiedResolver,
				Type:        util.BindOutput(account.OwnershipIsVerifiedResponse{}, "Get is verified ", nil),
			},
		},
	}))

	(*rootQueries)["ownership"] = &graphql.Field{
		Type: query,
		Resolve: func(p graphql.ResolveParams) (interface{}, error) {
			return query, nil
		},
	}

	mutation := graphql.NewObject(middleware.GraphqlMiddlewareObject(graphql.ObjectConfig{
		Name:        "OwnershipMutation",
		Description: "Ownership mutation root",
		Fields: graphql.Fields{
			"create": &graphql.Field{
				Description: "Create verification",
				Args:        util.BindArguments(account.CreateOwnershipVerificationRequest{}),
				Resolve:     CreateOwnershipVerificationResolver,
				Type:        util.BindOutput(account.CreateOwnershipVerificationResponse{}, "Create verification", nil),
			},
		},
	}))

	(*rootMutations)["ownership"] = &graphql.Field{
		Type: mutation,
		Resolve: func(p graphql.ResolveParams) (interface{}, error) {
			return mutation, nil
		},
	}
}

func OwnershipIsVerifiedResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value(ws.CtxUser).(*auth.AuthResponse)
	if user == nil {
		return nil, errs.ErrNoUser
	}

	request := &account.OwnershipIsVerifiedRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}

	resp, err := connections.AccountService.OwnershipIsVerified(params.Context, request)
	if err != nil {
		return nil, err
	}
	return resp, nil
}

func CreateOwnershipVerificationResolver(params graphql.ResolveParams) (interface{}, error) {
	user := params.Context.Value(ws.CtxUser).(*auth.AuthResponse)
	if user == nil {
		return nil, errs.ErrNoUser
	}

	request := &account.CreateOwnershipVerificationRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}

	request.AccountId = user.GetAccount().GetIdValue()
	resp, err := connections.AccountService.CreateOwnershipVerification(params.Context, request)
	if err != nil {
		return nil, err
	}
	return resp, nil
}
