package rest

import (
	"context"
	"crypto/sha256"
	"net/http"
	"strings"

	"gitlab.mashplace.com/mashplace/backend/common/generic/log"
	"gitlab.mashplace.com/mashplace/backend/gateway/impl/connections"
	signature "gitlab.mashplace.com/mashplace/backend/signature/api"
)

// Callback from Hellosign API

const (
	MultipartMemoryBuffer = (1 << 20) * 128
)

func HellosignHandler(req *http.Request) (int, string, error) {
	err := req.ParseMultipartForm(MultipartMemoryBuffer)
	if err != nil {
		return Err500Result(err)
	}

	jsonList, ok := req.MultipartForm.Value["json"]
	if !ok {
		return Err500Result(err)
	}
	jsonStr := strings.Join(jsonList, "")

	log.Info(jsonStr)

	data := []byte(jsonStr)

	h := sha256.New()

	_, _ = h.Write(data)

	actual := h.Sum(nil)

	stream, err := connections.SignatureService.ServiceSigningEvent(context.Background())
	if err != nil {
		return Err500Result(err)
	}

	err = stream.Send(&signature.ServiceSigningEventRequest{
		Event: &signature.ServiceSigningEventRequest_Init{
			Init: &signature.SigningEventInit{
				Sha256: actual,
			},
		},
	})
	if err != nil {
		return Err500Result(err)
	}

	for len(data) > 0 {
		var trans []byte
		if len(data) > signature.ChunkSize {
			trans = data[:signature.ChunkSize]
			data = data[signature.ChunkSize:]
		} else {
			trans = data
			data = nil
		}
		err = stream.Send(&signature.ServiceSigningEventRequest{
			Event: &signature.ServiceSigningEventRequest_Chunk{
				Chunk: &signature.SigningEventChunk{
					Data: trans,
				},
			},
		})

		if err != nil {
			return Err500Result(err)
		}
	}

	err = stream.CloseSend()
	if err != nil {
		return Err500Result(err)
	}

	return http.StatusOK, "Hello API Event Received", nil // required by hellosign protocol
}
