package rest

import (
	"io"
	"net/http"

	"gitlab.mashplace.com/mashplace/backend/common/generic/log"
	"gitlab.mashplace.com/mashplace/backend/gateway/impl/connections"
	identity "gitlab.mashplace.com/mashplace/backend/identity/api"
)

var RentprofileHandler = func() *http.ServeMux {
	m := http.NewServeMux()

	m.HandleFunc("/status", rentprofileHandlerStatus)
	m.HandleFunc("/result", rentprofileHandlerResult)

	return m
}()

func rentprofileHandlerStatus(w http.ResponseWriter, r *http.Request) {
	bs, err := io.ReadAll(r.Body)
	if err != nil {
		log.WithError(err).Error("reading rentprofile status request body")
		http.Error(w, err.Error(), http.StatusBadRequest)

		return
	}

	_, err = connections.IdentityService.WebhookStatus(r.Context(), &identity.WebhookStatusRequest{
		Data: bs,
	})
	if err != nil {
		log.WithError(err).Errorf("processing rentprofile status request body: %s", string(bs))
		http.Error(w, err.Error(), http.StatusInternalServerError)

		return
	}
}

func rentprofileHandlerResult(w http.ResponseWriter, r *http.Request) {
	bs, err := io.ReadAll(r.Body)
	if err != nil {
		log.WithError(err).Error("reading rentprofile result request body")
		http.Error(w, err.Error(), http.StatusBadRequest)

		return
	}

	_, err = connections.IdentityService.WebhookResult(r.Context(), &identity.WebhookResultRequest{
		Data: bs,
	})
	if err != nil {
		log.WithError(err).Errorf("processing rentprofile result request body: %s", string(bs))
		http.Error(w, err.Error(), http.StatusInternalServerError)

		return
	}
}
